@@ -80,17 +80,22 @@ impl CmdChildren {
             CmdChild::Proc {
                 mut child, stderr, ..
             } => {
+                let polling_stderr = stderr.map(CmdChild::log_stderr_output);
                 if let Some(stdout) = child.stdout.take() {
                     f(Box::new(stdout));
                     let _ = child.kill();
                 }
-                CmdChild::log_stderr_output(stderr);
+                if let Some(polling_stderr) = polling_stderr {
+                    let _ = polling_stderr.join();
+                }
             }
             CmdChild::ThreadFn { .. } => {
                 panic!("should not wait pipe on thread");
             }
             CmdChild::SyncFn { stderr, stdout, .. } => {
-                CmdChild::log_stderr_output(stderr);
+                if let Some(stderr) = stderr {
+                    let _ = CmdChild::log_stderr_output(stderr).join();
+                }
                 if let Some(stdout) = stdout {
                     f(Box::new(stdout));
                 }
@@ -140,9 +145,12 @@ impl CmdChild {
                 cmd,
                 ignore_error,
             } => {
+                let polling_stderr = stderr.map(CmdChild::log_stderr_output);
                 let status = child.wait()?;
-                Self::log_stderr_output(stderr);
-                Self::print_stdout_output(child.stdout.take());
+                if let Some(polling_stderr) = polling_stderr {
+                    Self::wait_stderr_thread(&cmd, polling_stderr)?;
+                }
+                Self::print_stdout_output(&mut child.stdout);
                 if !ignore_error && !status.success() && (is_last || pipefail) {
                     return Err(Self::status_to_io_error(
                         status,
@@ -157,8 +165,13 @@ impl CmdChild {
                 ignore_error,
                 ..
             } => {
+                let polling_stderr = stderr.map(CmdChild::log_stderr_output);
                 let status = child.join();
-                Self::log_stderr_output(stderr);
+                let polling_ret = if let Some(polling_stderr) = polling_stderr {
+                    Self::wait_stderr_thread(&cmd, polling_stderr)
+                } else {
+                    Ok(())
+                };
                 if ignore_error {
                     return Ok(());
                 }
@@ -173,17 +186,39 @@ impl CmdChild {
                     }
                     Ok(result) => {
                         check_result(result)?;
+                        if is_last || pipefail {
+                            return polling_ret;
+                        }
                     }
                 }
             }
-            CmdChild::SyncFn { stdout, stderr, .. } => {
-                Self::log_stderr_output(stderr);
-                Self::print_stdout_output(stdout);
+            CmdChild::SyncFn {
+                mut stdout,
+                stderr,
+                cmd,
+                ..
+            } => {
+                if let Some(stderr) = stderr {
+                    Self::wait_stderr_thread(&cmd, Self::log_stderr_output(stderr))?;
+                }
+                Self::print_stdout_output(&mut stdout);
             }
         }
         Ok(())
     }
 
+    fn wait_stderr_thread(cmd: &str, thread: JoinHandle<()>) -> CmdResult {
+        let join_ret = thread.join();
+        if let Err(e) = join_ret {
+            Err(Error::new(
+                ErrorKind::Other,
+                format!("{} stderr polling thread exited with error: {:?}", cmd, e),
+            ))
+        } else {
+            Ok(())
+        }
+    }
+
     fn wait_with_output(self) -> Result<Vec<u8>> {
         match self {
             CmdChild::Proc {
@@ -192,8 +227,11 @@ impl CmdChild {
                 stderr,
                 ignore_error,
             } => {
+                let polling_stderr = stderr.map(CmdChild::log_stderr_output);
                 let output = child.wait_with_output()?;
-                Self::log_stderr_output(stderr);
+                if let Some(polling_stderr) = polling_stderr {
+                    Self::wait_stderr_thread(&cmd, polling_stderr)?;
+                }
                 if !ignore_error && !output.status.success() {
                     return Err(Self::status_to_io_error(
                         output.status,
@@ -203,11 +241,37 @@ impl CmdChild {
                     Ok(output.stdout)
                 }
             }
-            CmdChild::ThreadFn { cmd, .. } => {
-                panic!("{} thread should not be waited for output", cmd);
+            CmdChild::ThreadFn {
+                cmd,
+                stdout,
+                stderr,
+                child,
+                ..
+            } => {
+                let polling_stderr = stderr.map(CmdChild::log_stderr_output);
+                // simulate process's wait_with_output() API
+                let buf = if let Some(mut out) = stdout {
+                    let mut buf = vec![];
+                    out.read_to_end(&mut buf)?;
+                    buf
+                } else {
+                    vec![]
+                };
+                child.join().unwrap()?;
+                if let Some(polling_stderr) = polling_stderr {
+                    Self::wait_stderr_thread(&cmd, polling_stderr)?;
+                }
+                Ok(buf)
             }
-            CmdChild::SyncFn { stdout, stderr, .. } => {
-                Self::log_stderr_output(stderr);
+            CmdChild::SyncFn {
+                cmd,
+                stdout,
+                stderr,
+                ..
+            } => {
+                if let Some(stderr) = stderr {
+                    Self::wait_stderr_thread(&cmd, Self::log_stderr_output(stderr))?;
+                }
                 if let Some(mut out) = stdout {
                     let mut buf = vec![];
                     out.read_to_end(&mut buf)?;
@@ -218,7 +282,7 @@ impl CmdChild {
         }
     }
 
-    fn print_stdout_output(stdout: Option<impl Read>) {
+    fn print_stdout_output(stdout: &mut Option<impl Read>) {
         if let Some(stdout) = stdout {
             BufReader::new(stdout)
                 .lines()
@@ -227,13 +291,13 @@ impl CmdChild {
         }
     }
 
-    fn log_stderr_output(stderr: Option<impl Read>) {
-        if let Some(stderr) = stderr {
+    fn log_stderr_output(stderr: PipeReader) -> JoinHandle<()> {
+        std::thread::spawn(move || {
             BufReader::new(stderr)
                 .lines()
                 .filter_map(|line| line.ok())
-                .for_each(|line| info!("{}", line));
-        }
+                .for_each(|line| info!("{}", line))
+        })
     }
 
     fn status_to_io_error(status: ExitStatus, command: &str) -> Error {
