@@ -99,34 +99,32 @@ impl GroupCmds {
 
     pub fn run_cmd(&mut self) -> CmdResult {
         for cmds in self.group_cmds.iter_mut() {
-            if let Err(err) = cmds.run_cmd(&mut self.current_dir) {
-                error!("Running {} failed, Error: {}", cmds.get_full_cmds(), err);
-                return Err(err);
+            if let Err(e) = cmds.run_cmd(&mut self.current_dir) {
+                error!("Running {} failed, Error: {}", cmds.get_full_cmds(), e);
+                return Err(e);
             }
         }
         Ok(())
     }
 
     pub fn run_fun(&mut self) -> FunResult {
+        // run previous commands
         let mut last_cmd = self.group_cmds.pop().unwrap();
         self.run_cmd()?;
         // run last function command
         let ret = last_cmd.run_fun(&mut self.current_dir);
-        if let Err(e) = ret {
-            let full_cmds = last_cmd.get_full_cmds();
-            error!("Running {} failed, Error: {}", full_cmds, e);
-            Err(e)
-        } else {
-            ret
+        if let Err(ref e) = ret {
+            error!("Running {} failed, Error: {}", last_cmd.get_full_cmds(), e);
         }
+        ret
     }
 
     pub fn spawn(mut self, with_output: bool) -> Result<CmdChildren> {
         assert_eq!(self.group_cmds.len(), 1);
         let mut cmds = self.group_cmds.pop().unwrap();
         let ret = cmds.spawn(&mut self.current_dir, with_output);
-        if let Err(ref err) = ret {
-            error!("Spawning {} failed, Error: {}", cmds.get_full_cmds(), err);
+        if let Err(ref e) = ret {
+            error!("Spawning {} failed, Error: {}", cmds.get_full_cmds(), e);
         }
         ret
     }
@@ -161,16 +159,16 @@ impl Cmds {
         // spawning all the sub-processes
         let mut children: Vec<CmdChild> = Vec::new();
         let len = self.cmds.len();
-        let mut last_pipe_in = None;
+        let mut prev_pipe_in = None;
         for (i, cmd_opt) in self.cmds.iter_mut().enumerate() {
             let mut cmd = cmd_opt.take().unwrap();
             if i != len - 1 {
                 // not the last, update redirects
                 let (pipe_reader, pipe_writer) = os_pipe::pipe()?;
-                cmd.setup_redirects(&mut last_pipe_in, Some(pipe_writer))?;
-                last_pipe_in = Some(pipe_reader);
+                cmd.setup_redirects(&mut prev_pipe_in, Some(pipe_writer))?;
+                prev_pipe_in = Some(pipe_reader);
             } else {
-                cmd.setup_redirects(&mut last_pipe_in, None)?;
+                cmd.setup_redirects(&mut prev_pipe_in, None)?;
             }
             let child = cmd.spawn(current_dir, with_output)?;
             children.push(child);
@@ -353,22 +351,22 @@ impl Cmd {
                 stdin: if let Some(redirect_in) = self.stdin_redirect.take() {
                     redirect_in
                 } else {
-                    CmdIn::CmdPipe(os_pipe::dup_stdin()?)
+                    CmdIn::Pipe(os_pipe::dup_stdin()?)
                 },
                 stdout: if let Some(redirect_out) = self.stdout_redirect.take() {
                     redirect_out
                 } else {
-                    CmdOut::CmdPipe(os_pipe::dup_stdout()?)
+                    CmdOut::Pipe(os_pipe::dup_stdout()?)
                 },
                 stderr: if let Some(redirect_err) = self.stderr_redirect.take() {
                     redirect_err
                 } else {
-                    CmdOut::CmdPipe(os_pipe::dup_stderr()?)
+                    CmdOut::Pipe(os_pipe::dup_stderr()?)
                 },
             };
 
             let internal_cmd = CMD_MAP.lock().unwrap()[&arg0];
-            if pipe_out {
+            if pipe_out || with_output {
                 let handle = std::thread::spawn(move || internal_cmd(&mut env));
                 Ok(CmdChild::ThreadFn {
                     child: handle,
@@ -465,57 +463,57 @@ impl Cmd {
     ) -> CmdResult {
         // set up error pipe
         let (pipe_reader, pipe_writer) = os_pipe::pipe()?;
-        self.stderr_redirect = Some(CmdOut::CmdPipe(pipe_writer));
+        self.stderr_redirect = Some(CmdOut::Pipe(pipe_writer));
         self.stderr_logging = Some(pipe_reader);
 
         if let Some(pipe) = pipe_out {
-            self.stdout_redirect = Some(CmdOut::CmdPipe(pipe));
+            self.stdout_redirect = Some(CmdOut::Pipe(pipe));
         } else if self.in_cmd_map {
             // set up stdout pipe
             let (pipe_reader, pipe_writer) = os_pipe::pipe()?;
-            self.stdout_redirect = Some(CmdOut::CmdPipe(pipe_writer));
+            self.stdout_redirect = Some(CmdOut::Pipe(pipe_writer));
             self.stdout_logging = Some(pipe_reader);
         }
 
         if let Some(pipe) = pipe_in.take() {
-            self.stdin_redirect = Some(CmdIn::CmdPipe(pipe));
+            self.stdin_redirect = Some(CmdIn::Pipe(pipe));
         }
 
         for redirect in self.redirects.iter() {
             match redirect {
                 Redirect::FileToStdin(path) => {
                     self.stdin_redirect = Some(if path == Path::new("/dev/null") {
-                        CmdIn::CmdNull
+                        CmdIn::Null
                     } else {
-                        CmdIn::CmdFile(Self::open_file(path, true, false)?)
+                        CmdIn::File(Self::open_file(path, true, false)?)
                     });
                 }
                 Redirect::StdoutToStderr => {
                     if let Some(ref redirect) = self.stderr_redirect {
                         self.stdout_redirect = Some(redirect.try_clone()?);
                     } else {
-                        self.stdout_redirect = Some(CmdOut::CmdPipe(os_pipe::dup_stderr()?));
+                        self.stdout_redirect = Some(CmdOut::Pipe(os_pipe::dup_stderr()?));
                     }
                 }
                 Redirect::StderrToStdout => {
                     if let Some(ref redirect) = self.stdout_redirect {
                         self.stderr_redirect = Some(redirect.try_clone()?);
                     } else {
-                        self.stderr_redirect = Some(CmdOut::CmdPipe(os_pipe::dup_stdout()?));
+                        self.stderr_redirect = Some(CmdOut::Pipe(os_pipe::dup_stdout()?));
                     }
                 }
                 Redirect::StdoutToFile(path, append) => {
                     self.stdout_redirect = Some(if path == Path::new("/dev/null") {
-                        CmdOut::CmdNull
+                        CmdOut::Null
                     } else {
-                        CmdOut::CmdFile(Self::open_file(path, false, *append)?)
+                        CmdOut::File(Self::open_file(path, false, *append)?)
                     });
                 }
                 Redirect::StderrToFile(path, append) => {
                     self.stderr_redirect = Some(if path == Path::new("/dev/null") {
-                        CmdOut::CmdNull
+                        CmdOut::Null
                     } else {
-                        CmdOut::CmdFile(Self::open_file(path, false, *append)?)
+                        CmdOut::File(Self::open_file(path, false, *append)?)
                     });
                 }
             }
